-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

--constants
local minClaw = -0.2
local maxClaw = 1.0

--sound vars
local hydroOnePV = 0
local hydroTwoPV = 0

--arms
local lastFirstArmValue = 0
local lastSecondArmValue = 0
local firstArmInput = 0
local secondArmInput = 0

local function ToggleClose()
    local curValue = electrics.values['claw']
    if curValue < 0 then
        electrics.values['claw'] = maxClaw
    else
        electrics.values['claw'] = minClaw
    end
end

local function SetFirstArm(value)
    firstArmInput = value
end

local function SetSecondArm(value)
    secondArmInput = value
end

local function init()
    print("onextload")
    electrics.values['cranearm1'] = 0
	electrics.values['cranearm2'] = 0
    electrics.values['claw'] = minClaw

    electrics.values['hydroOnePV'] = 0
    electrics.values['hydroTwoPV'] = 0

    hydroOnePV = 0
    hydroTwoPV = 0
    
    lastFirstArmValue = 0
    lastSecondArmValue = 0

    SetFirstArm(0)
    SetSecondArm(0)
end

local function reset()
	init()
end

local function clamp(val, min, max)
    if val < min then return min end
    if val > max then return max end
    return val
end

local function signWithTolerance(num, tolerance)
    if math.abs(num) < tolerance then return 0 end
    if num > 0 then return 1 end
    if num < 0 then return -1 end
    return 0
end

local function updateGFX(dt)
        --calculate extend/retract rate
		local firstRate = firstArmInput * 0.075 * dt
		local secondRate = secondArmInput * 0.125 * dt
    
        --set rate, and clamp
	    electrics.values['cranearm1'] = clamp(lastFirstArmValue + firstRate, -0.05, 0.55)
	    electrics.values['cranearm2'] = clamp(lastSecondArmValue + secondRate, -0.1, 0.56)

        --get  difference between last set value
        local realFirstRate = (electrics.values['cranearm1'] - lastFirstArmValue) / dt
        local realSecondRate = (electrics.values['cranearm2'] - lastSecondArmValue) / dt

        --update last values
        lastFirstArmValue = electrics.values['cranearm1']
        lastSecondArmValue = electrics.values['cranearm2']

        --set sound vars
        local realFirstRateSign = signWithTolerance(realFirstRate, 0.05)
        local realSecondRateSign = signWithTolerance(realSecondRate, 0.05)

        local onePVDecay = -1  + (1 - math.abs(realFirstRateSign)) + (1 * (1 - math.abs(realFirstRateSign)))
        local twoPVDecay = -1  + (1 - math.abs(realSecondRateSign)) + (1 * (1 - math.abs(realSecondRateSign)))

        --on hydro "ramp up" we make it go faster
        if onePVDecay < 0 then onePVDecay = onePVDecay * 3 end
        if twoPVDecay < 0 then twoPVDecay = twoPVDecay * 3 end

        hydroOnePV = clamp(hydroOnePV - (onePVDecay * dt * 1), 0, 0.99)
        hydroTwoPV = clamp(hydroTwoPV - (twoPVDecay * dt * 1), 0, 0.99)

        electrics.values['hydroOnePV'] = hydroOnePV
        electrics.values['hydroTwoPV'] = hydroTwoPV
end

-- public interface
M.onInit      = init
M.onReset     = reset
M.updateGFX = updateGFX

M.SetFirstArm = SetFirstArm
M.SetSecondArm = SetSecondArm
M.ToggleClose = ToggleClose

return M